// reads raw data from run_xxx.txt assigned to link indata
// writes ascii tuple file: ntuple.ascii using nout
#include <iostream>
#include <iomanip>
#include <fstream>		// needed by  ofstream to open output file
#include "histogram.h"
#include <string>

#define TAB '\t'

using namespace std;

int main(int argc, char** argv)	{

	int lineID, hdata, ddata;
	int chunkStart=3, chunkSize, chunkEnd=0xFFFF;
	int feraID;
	int lineTotalLineCount = 0;
	int fileLineCount = 0;
	int linesInChunk=0;
	int bufferNumber=0, bufferCount=0;
	string titleString;
	int ptrToNext=1;
	bool notTheFirst=false, ptrData=false, firstTime = true ;
	int seconds,useconds;
	double relativeTime = 0.0, absoluteTime = 0.0, firstAbsoluteTime = 0.0;
	double timePair[2];
	double pulserDelay = 3.0;	// 0.0, 3.0 nsec
	double pulserPeriod = 0.5;	// 2.0, 1.0, 0.5 msec
	int ntupleCount=0;
	int countNumbTS=-1;
	double timeNextPair[2];
	int runNumber=0;
	//unsigned int event[18];
	unsigned int event[28];
	int eventIndex=-1, eventCount=0;
	int ad114value1=0, f4300bCh1=0;
	int ad114value2=0, f4300bCh2=0;
	int errorType1count = 0;


/*
	0	adc channel 0
	1	adc channel 1
	15	adc channel 15
	16	AD114 header 8801
	17	AD114 data
	18	AD114 header 8802
	19	AD114 data
 */

	cout << "Enter Run Description parameters: " << endl;
	cout <<"runNumber  pulserDelay  pulserPeriod " << endl;
	cin >> runNumber >> pulserDelay >> pulserPeriod;


	ifstream in("indata");


	// 1st line is text not data
	getline(in,titleString);
	fileLineCount++;
	
	cout << "Title: " << titleString << endl;
	cout << "DEBUG:\tLine\tlineID\thdata\tddata" << endl;

   // open an output stream to make file of ascii nutples for ROOT
   ofstream nout("ntuple.ascii");
   if(!nout)
   {
     cout << "Cannot open output file.\n";
     return 1;
   }


   for (;;) {
	

	lineID = hdata = ddata = 0;
	seconds = useconds = 0;
	in >> lineID;
	if (lineID == -1) { in >> seconds >> useconds; fileLineCount++; }
	if (lineID == -2) { in >> seconds >> useconds; fileLineCount++; }
	if (lineID >=  0) { in >> hex >> hdata >> dec >> ddata; fileLineCount++; }
	if ( eventCount%100 == 0 ) {
		if (lineID < 0 ) cout << fileLineCount << TAB << lineID <<TAB
					<< seconds<<TAB<<useconds<<TAB<< endl;
		if (lineID >= 0) cout << fileLineCount << TAB << lineID <<TAB
					<<hex<<hdata<<TAB<<dec<<ddata<< endl;
	}

	// detect an EOF
	if ( in.eof() != 0 ) {
		cout << "Found end of file..." << endl;
		break;
	}

	ptrData=true;
	if ( lineID == ptrToNext ) {
		ptrData = false;
		ptrToNext = ptrToNext + (hdata & 0x0FFF) + 1;
		cout << "At fileLineCount="<<fileLineCount<<"  lineID = " << lineID 
			<< TAB << "Found ptrData = " << ptrData << TAB 
			<< "ptrToNext = " << ptrToNext << endl;
		}
	if ( hdata == 0xFFFF ) {
			cout << "At fileLineCount="<<fileLineCount<<"  lineID = "<<lineID 
					<< "Found a FFFF tag... reset ptrToNext to 1." << endl; 
			 ptrToNext = 1; 
	}
	
	if ( lineID < -2 ){cout << "This should never happen! Exit with error" << endl; return 1;}
	if ( lineID == -1 || lineID == -2 ) {
		cout << "Found a "<< lineID << " TimeStamp: seconds=" << seconds 
			<< " useconds=" << useconds << " firstTime = " << firstTime << endl;
		absoluteTime = seconds + useconds/1.0e+6;
		cout << "Converts to absoluteTime = " << absoluteTime << endl;
		if ( firstTime ) firstAbsoluteTime = absoluteTime;
		firstTime = false;
		relativeTime = absoluteTime - firstAbsoluteTime;
		cout << "Converts to relative time = " << relativeTime 
			<< " using firstAbsoluteTime = " << firstAbsoluteTime << endl;
		if ( lineID == -1 ) timePair[0] = relativeTime;
		if ( lineID == -2 ) timePair[1] = relativeTime;
		countNumbTS++;
		if ( (countNumbTS%4 == 0) ) timeNextPair[0] = relativeTime;
		if ( (countNumbTS%4 == 2) ) timeNextPair[1] = relativeTime;
		//out << lineID << TAB << seconds << TAB << useconds << endl;
		//out << lineID << TAB << relativeTime << endl;
	}


	if ( lineID >= 0 && ptrData ) {
	   //in >> lineID >> hex >> hdata >> dec >> ddata;
	   lineTotalLineCount++;
	   linesInChunk++;
	   if ( lineTotalLineCount <= 20 ) cout << "Read a line " << lineID << " " 
					<< hex << hdata << " " 
					<< dec << ddata << endl;

	   if ( lineID == 1 ) cout << "Size = " << ddata << endl;

	   if ( lineID == 0 || lineID == 1 || hdata == 0x0003 || hdata == 0x8002
		|| hdata == 0xFFFF ) cout << "DEBUG:\t" << lineTotalLineCount << "\t"
					<< lineID << "\t"
					<< hex << hdata << "\t"
                                        << dec << ddata << endl;
	   if ( hdata == 0x14FD || hdata == 0x01B2 ) cout << "DEBUG:\t"<<lineTotalLineCount << "\t"
                                        << lineID << "\t"
                                        << hex << hdata << "\t"
                                        << dec << ddata << "\tadditional" << endl;
	

           //ixbin=khist( (double) x, alow, ahgh, 16);
	   // histogram X distribution for all ntuples

	   if ( lineID > 1 && hdata != 0xFFFF ) {
		eventIndex++;
		event[eventIndex] = ddata;
		if ( eventIndex == 19 ) {
			eventIndex = -1;
			eventCount++;
			if ( event[16] != 0x8801 || event[18] != 0x8802) {
				cout << "Event builder error at  fileLineCount = " << fileLineCount 
					<< " lineID = " << lineID << endl;
				for ( int i1=0; i1<20; i1++) cout << i1 <<TAB<< hex << event[i1] 
								<<TAB << dec << event[i1]<< endl;
				// return 1;
				// attempt to fix error type 1
				errorType1count++;
				cout << "attempting to fixe error " << errorType1count << endl;
				event[0] = event[16];
				event[1] = event[17];
				event[2] = event[18];
				event[3] = event[19];
				eventIndex = 3;	// reset pointer into event[]
			} else {
		
			ad114value1 = event[17];
			ad114value2 = event[19];
			f4300bCh1  = event[1];
			f4300bCh2  = event[3];
			if ( eventCount%100 == 0) {
			cout <<"Found complete event ad114value1="<< ad114value1 
			<< " f4300bCh1=" << f4300bCh1 
			<< " ad114value2="<< ad114value2
			<< " f4300bCh2=" << f4300bCh2 
			<< endl;
			}
			//nout << ad114value1 << TAB << f4300bCh1 << TAB
			//	<< ad114value2 << TAB << f4300bCh2 << endl;
			ntupleCount++;
			nout << runNumber << TAB << ntupleCount << TAB
				<< timePair[0] << TAB << timePair[1] << TAB 
				<< pulserDelay << TAB << pulserPeriod << TAB
				<< ad114value1 << TAB << f4300bCh1 << TAB
				<< ad114value2 << TAB << f4300bCh2 << endl;
			}
		}
	   }
		

	   if ( lineID == 0 ) bufferCount++;
	   // detect change to new buffer
	   if ( lineID == 0  && notTheFirst ) {
		cout << "Found transition at lineTotalLineCount = " 
			<< lineTotalLineCount << " lines in a Chunk = " << linesInChunk << endl;
		linesInChunk = 0;
		bufferNumber++;
	   }
	   notTheFirst = true;

	} // end of 'if ( lineID >= 0 )'
   } // end of forever loop

	// output histograms.
	cout << "Found " << lineTotalLineCount << " lines."  << endl;
	cout << "bufferCount     = " <<  bufferCount << endl;
	cout << "eventCount      = " << eventCount << endl;
	cout << "errorType1count = " << errorType1count << endl;
	cout << "ntupleCount     = " << ntupleCount << endl;

	nout.close();

	return 0;

}
